@@ -1,4 +1,4 @@
-# Ultralytics YOLO 🚀, GPL-3.0 license
+# Ultralytics YOLO 🚀, AGPL-3.0 license
 
 from pathlib import Path
 
@@ -207,7 +207,6 @@ def test_predict_callback_and_setup():
 def test_result():
     model = YOLO('yolov8n-pose.pt')
     res = model([SOURCE, SOURCE])
-    res[0].plot(show_conf=False)  # raises warning
     res[0].plot(conf=True, boxes=False)
     res[0].plot(pil=True)
     res[0] = res[0].cpu().numpy()
@@ -215,11 +214,10 @@ def test_result():
 
     model = YOLO('yolov8n-seg.pt')
     res = model([SOURCE, SOURCE])
-    res[0].plot(show_conf=False)  # raises warning
     res[0].plot(conf=True, boxes=False, masks=True)
     res[0].plot(pil=True)
     res[0] = res[0].cpu().numpy()
-    print(res[0].path, res[0].masks.masks)
+    print(res[0].path, res[0].masks.data)
 
     model = YOLO('yolov8n.pt')
     res = model(SOURCE)
