@@ -1,4 +1,4 @@
-# Ultralytics YOLO 🚀, GPL-3.0 license
+# Ultralytics YOLO 🚀, AGPL-3.0 license
 import contextlib
 import glob
 import inspect
@@ -47,13 +47,13 @@ def check_imgsz(imgsz, stride=32, min_dim=1, max_dim=2, floor=0):
     stride, update it to the nearest multiple of the stride that is greater than or equal to the given floor value.
 
     Args:
-        imgsz (int or List[int]): Image size.
+        imgsz (int) or (cList[int]): Image size.
         stride (int): Stride value.
         min_dim (int): Minimum number of dimensions.
         floor (int): Minimum allowed value for image size.
 
     Returns:
-        List[int]: Updated image size.
+        (List[int]): Updated image size.
     """
     # Convert stride to integer if it is a tensor
     stride = int(stride.max() if isinstance(stride, torch.Tensor) else stride)
@@ -106,7 +106,7 @@ def check_version(current: str = '0.0.0',
         verbose (bool): If True, print warning message if minimum version is not met.
 
     Returns:
-        bool: True if minimum version is met, False otherwise.
+        (bool): True if minimum version is met, False otherwise.
     """
     current, minimum = (pkg.parse_version(x) for x in (current, minimum))
     result = (current == minimum) if pinned else (current >= minimum)  # bool
@@ -126,9 +126,10 @@ def check_latest_pypi_version(package_name='ultralytics'):
         package_name (str): The name of the package to find the latest version for.
 
     Returns:
-        str: The latest version of the package.
+        (str): The latest version of the package.
     """
-    response = requests.get(f'https://pypi.org/pypi/{package_name}/json')
+    requests.packages.urllib3.disable_warnings()  # Disable the InsecureRequestWarning
+    response = requests.get(f'https://pypi.org/pypi/{package_name}/json', verify=False)
     if response.status_code == 200:
         return response.json()['info']['version']
     return None
@@ -139,7 +140,7 @@ def check_pip_update_available():
     Checks if a new version of the ultralytics package is available on PyPI.
 
     Returns:
-        bool: True if an update is available, False otherwise.
+        (bool): True if an update is available, False otherwise.
     """
     if ONLINE and is_pip_package():
         with contextlib.suppress(Exception):
@@ -196,7 +197,16 @@ def check_python(minimum: str = '3.7.0') -> bool:
 
 @TryExcept()
 def check_requirements(requirements=ROOT.parent / 'requirements.txt', exclude=(), install=True, cmds=''):
-    # Check installed dependencies meet YOLOv5 requirements (pass *.txt file or list of packages or single package str)
+    """
+    Check if installed dependencies meet YOLOv8 requirements and attempt to auto-update if needed.
+
+    Args:
+        requirements (Union[Path, str, List[str]]): Path to a requirements.txt file, a single package requirement as a
+            string, or a list of package requirements as strings.
+        exclude (Tuple[str]): Tuple of package names to exclude from checking.
+        install (bool): If True, attempt to auto-update packages that don't meet requirements.
+        cmds (str): Additional commands to pass to the pip install command when auto-updating.
+    """
     prefix = colorstr('red', 'bold', 'requirements:')
     check_python()  # check python version
     file = None
@@ -208,8 +218,8 @@ def check_requirements(requirements=ROOT.parent / 'requirements.txt', exclude=()
     elif isinstance(requirements, str):
         requirements = [requirements]
 
-    s = ''
-    n = 0
+    s = ''  # console string
+    n = 0  # number of packages updates
     for r in requirements:
         try:
             pkg.require(r)
@@ -225,7 +235,7 @@ def check_requirements(requirements=ROOT.parent / 'requirements.txt', exclude=()
         LOGGER.info(f"{prefix} YOLOv8 requirement{'s' * (n > 1)} {s}not found, attempting AutoUpdate...")
         try:
             assert is_online(), 'AutoUpdate skipped (offline)'
-            LOGGER.info(subprocess.check_output(f'pip install {s} {cmds}', shell=True).decode())
+            LOGGER.info(subprocess.check_output(f'pip install --no-cache {s} {cmds}', shell=True).decode())
             s = f"{prefix} {n} package{'s' * (n > 1)} updated per {file or requirements}\n" \
                 f"{prefix} ⚠️ {colorstr('bold', 'Restart runtime or rerun command for updates to take effect')}\n"
             LOGGER.info(s)
@@ -234,7 +244,7 @@ def check_requirements(requirements=ROOT.parent / 'requirements.txt', exclude=()
 
 
 def check_suffix(file='yolov8n.pt', suffix='.pt', msg=''):
-    # Check file(s) for acceptable suffix
+    """Check file(s) for acceptable suffix."""
     if file and suffix:
         if isinstance(suffix, str):
             suffix = (suffix, )
@@ -245,7 +255,7 @@ def check_suffix(file='yolov8n.pt', suffix='.pt', msg=''):
 
 
 def check_yolov5u_filename(file: str, verbose: bool = True):
-    # Replace legacy YOLOv5 filenames with updated YOLOv5u filenames
+    """Replace legacy YOLOv5 filenames with updated YOLOv5u filenames."""
     if ('yolov3' in file or 'yolov5' in file) and 'u' not in file:
         original_file = file
         file = re.sub(r'(.*yolov5([nsmlx]))\.pt', '\\1u.pt', file)  # i.e. yolov5n.pt -> yolov5nu.pt
@@ -259,7 +269,7 @@ def check_yolov5u_filename(file: str, verbose: bool = True):
 
 
 def check_file(file, suffix='', download=True, hard=True):
-    # Search/download file (if necessary) and return path
+    """Search/download file (if necessary) and return path."""
     check_suffix(file, suffix)  # optional
     file = str(file).strip()  # convert to string and strip spaces
     file = check_yolov5u_filename(file)  # yolov5n -> yolov5nu
@@ -285,12 +295,12 @@ def check_file(file, suffix='', download=True, hard=True):
 
 
 def check_yaml(file, suffix=('.yaml', '.yml'), hard=True):
-    # Search/download YAML file (if necessary) and return path, checking suffix
+    """Search/download YAML file (if necessary) and return path, checking suffix."""
     return check_file(file, suffix, hard=hard)
 
 
 def check_imshow(warn=False):
-    # Check if environment supports image displays
+    """Check if environment supports image displays."""
     try:
         assert not any((is_colab(), is_kaggle(), is_docker()))
         cv2.imshow('test', np.zeros((1, 1, 3)))
@@ -305,6 +315,7 @@ def check_imshow(warn=False):
 
 
 def check_yolo(verbose=True, device=''):
+    """Return a human-readable YOLO software and hardware summary."""
     from ultralytics.yolo.utils.torch_utils import select_device
 
     if is_colab():
@@ -336,9 +347,10 @@ def git_describe(path=ROOT):  # path must be a directory
 
 
 def print_args(args: Optional[dict] = None, show_file=True, show_func=False):
-    # Print function arguments (optional args dict)
+    """Print function arguments (optional args dict)."""
+
     def strip_auth(v):
-        # Clean longer Ultralytics HUB URLs by stripping potential authentication information
+        """Clean longer Ultralytics HUB URLs by stripping potential authentication information."""
         return clean_url(v) if (isinstance(v, str) and v.startswith('http') and len(v) > 100) else v
 
     x = inspect.currentframe().f_back  # previous frame
